//api
const website = 'https://siddur.cn/';
//web resource
const urlPrex = 'https://cdn.siddur.cn/';

let libTree;
//接受ajax请求事件，ajax请求分两类：访问官网后台，或者通过寄主访问寄主的后台
chrome.runtime.onMessage.addListener(function(message, sender, sendResponse){
    const tabId = sender.tab.id;
    if(message.command){
        const {method, callback, ...params} = message.params;
        switch(message.command){
            case "cookie":
                chrome.cookies[method](params, (e) => {
                    sendResponse(e);
                });
                return true;
            case "storage":
                const {type, content} = params;
                chrome.storage[type][method](content, (e) => {
                    sendResponse(e);
                });
                return true;
        }
    }else{
        if(message.target){
            chrome.webNavigation.getAllFrames({tabId: tabId}, frames => {
                const frame = frames.find(f => f.parentFrameId <=0 && f.url.startsWith(message.target));
                if(frame){
                    try{
                        chrome.tabs.sendMessage(tabId, message, {frameId: frame.frameId}, function(response){
                            chrome.tabs.sendMessage(tabId, {
                                requestId: message.requestId,
                                result: response
                            }, {frameId: sender.frameId});
                        })
                    }catch(e){
                        console.log(e);
                    }
                }
            })
            chrome.tabs.query({url: message.target}, function(tabs){
                const tab = tabs[0];
                if(tab){
                    chrome.tabs.sendMessage(tab.id, message, function(response){
                        chrome.tabs.sendMessage(sender.tab.id, {
                            requestId: message.requestId,
                            result: response
                        });
                    })
                }
            })
        }else{
            get(message.url, function(response){
                chrome.tabs.sendMessage(tabId, {
                    requestId: message.requestId,
                    result: response
                });
            })
        }
    }
})
chrome.tabs.onUpdated.addListener(function (tabId, statusObj, obj) {
    if(statusObj.status == 'complete'){
        fetchSiteMap({
            url:obj.url,
            tabId:tabId,
            handle:"onUpdated"
        });
    }
})

//onBeforeNavigate -> onCommitted -> onDOMContentLoaded -> onCompleted
//inject document script
chrome.webNavigation.onCommitted.addListener(function(details){
    const tabId = details.tabId;
    chrome.tabs.executeScript(tabId, {
        code:  `
            window.urlPrex = '${urlPrex}';
            window.website = '${website}';
        `,
    });
    fetchSiteMap({...details,handle:"onCommitted"});
})
var matchAndInjectText = function(scripts, inject){
    var jsArray = [];
    var cssArray = [];

    if(typeof scripts == "string" && scripts.includes('\n')){
        const arr = scripts.split('\n');
        arr.forEach(e=>{
            let ext = getExt(e);
            ext === "js" && jsArray.push(e);
            ext === "css" && cssArray.push(e);
        });
        inject(jsArray, cssArray);
    }
}
var matchAndInject = function(scripts, details, inject){
    var url = details.url;
    var jsArray = [];
    var cssArray = [];
    var cycleArray = function(libArray){
        var libItem = libArray.find(i => {
            if(typeof i.url == 'string'){
               return url.startsWith(i.url);
            }
            return i.url.some(j => url.startsWith(j));
        });
        if(libItem){
            libItem.js && libItem.js.forEach(j => jsArray.push(j));
            libItem.css && libItem.css.forEach(c => cssArray.push(c));
            libItem.file && libItem.file.forEach(f => {
                let ext = getExt(f);
                ext == 'js' && jsArray.push(f);
                ext == 'css' && cssArray.push(f);
            })
            if(libItem.children){
                libArray = libItem.children;
                cycleArray(libArray);
            }
        }
    }
    cycleArray(scripts);
    inject(jsArray, cssArray);
}

var injectContentCSS = function (tabId, file, frameId, domain = '') {
    if(file.indexOf('http') == -1){
        file = (domain || urlPrex) + file;
    }
    get(file, function (code) {
        chrome.tabs.insertCSS(tabId, {
            code: code,
            frameId: frameId
        });
    })
}

var injectDocumentCSS = function (tabId, file, frameId, domain = '') {
    if(file.indexOf('http') == -1){
        file = (domain || urlPrex) + file;
    }
    chrome.tabs.executeScript(tabId, {
        frameId: frameId,
        code: `
        (function () {
            var s = document.createElement('link');
            s.rel = "stylesheet";
            s.href = '${file}';
            s.onload = function () {
                this.remove();
            };
            if(document.head){
                if(document.head.firstChild){
                    (document.head).insertBefore(s, document.head.firstChild);
                }else{
                    (document.head).appendChild(s);
                }
            }
        })();
        `
    });
}

var injectContentScript = function (tabId, file, frameId, domain = '') {
    if(file.indexOf('http') == -1){
        file = (domain || urlPrex) + file;
    }
    get(file, function (code) {
        chrome.tabs.executeScript(tabId, {
            code: code,
            frameId: frameId
        });
    })
}

var injectDocumentScript = function(tabId, file, frameId, domain = ''){
    if(file.indexOf('http') == -1){
        file = (domain || urlPrex) + file;
    }
    fetch(file).then(resp => resp.text()).then(text => {
        chrome.tabs.executeScript(tabId, {
            frameId: frameId,
            code: `
                (function () {
                const b = new Blob([\`${text}\`], { type: 'text/javascript' });
                const u = URL.createObjectURL(b);
                var s = document.createElement('script');
                s.id = "abc";
                s.src = u;
                s.onload = function () {
                    //this.remove();
                };
                if(document.head){
                    if(document.head.firstChild){
                        (document.head).insertBefore(s, document.head.firstChild);
                    }else{
                    (document.head).appendChild(s);
                    }
                }
                })();
            `
        });
    })
}
//ajax get
var get = function (url, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open("GET", url, true);
    xhr.onreadystatechange = function () {
        if (xhr.readyState == 4) {
            callback(xhr.responseText);
        }
    }
    xhr.send();
}
//fetch map of site
var fetchSiteMap = function(details){
    let url = details.url;
    var t = (Date.now() + "").substring(0, 9);
    get(urlPrex + "plugins/sycm4/domain.map.js?t=" + t, function(maps){
        if(typeof maps === "string"){ maps = JSON.parse(maps)};
        if(maps instanceof Array){
            let filterList = maps.filter(e=>{
                let has = '';
                if(e.url instanceof Array){
                    has = e.url.find(_e => { return url.startsWith(_e)});
                }
                let a = e.url === "*" || has || !e.url;
                return a && details.handle === e.handle;
            });
             //获取子路径的方法
            fetchSubMap(filterList,(cList)=>{
                for (let i = 0; i < cList.length; i++) {
                    const e = cList[i];
                    let eList = [],domain = '';
                    if(e.url.includes("http")){
                        eList = e.url.split("/"); eList.pop();
                        domain = eList.join('/')+'/';
                    }else{
                        eList = e.url.split("/"); eList.pop();
                        domain = urlPrex + eList.join('/')+'/';
                    }
                    /* 整理code */
                    let code = e.code;
                    if(e.type === "json"){
                        let index = e.code.indexOf('=');
                        if(index > 0 && index < 10){
                            code =  e.code.substring(index + 1);
                        }
                        code = JSON.parse(code);
                    }else if(e.type === "conf"){
                        let arr = e.code.split('\n');
                        code = [];
                        arr.forEach(_e => {
                            if(_e.includes('=')){
                                let _arr = _e.split('=');
                                code.push({
                                    "url":_arr[0],
                                    "file":[_arr[1]]
                                })
                            }
                        });
                    }
                    if(!e.frame){
                        if(details.handle === "onCommitted"){
                            if(details.frameId === 0){
                                injectController[e.type](details,code,domain,e.target);
                            }
                        }else{
                            injectController[e.type](details,code,domain,e.target);
                        }
                    }else{
                        injectController[e.type](details,code,domain,e.target);
                    }
                }
            })
        }
    })
};

var injectController = {
    "conf":function(details,code,domain,target){
        jsonInject(details,code,domain,target);
    },
    "json":function(details,code,domain,target){
        code.documentScript && jsonInject(details,code.documentScript,domain,'document');
        code.contentScript && jsonInject(details,code.contentScript,domain,'content');
        if(!code.documentScript && !code.contentScript){
            jsonInject(details,code,domain,target);
        }
    },
    "text":function(details,code,domain,target){
        matchAndInjectText(code,(jsArray, cssArray)=>{
            jsArray.forEach(f => {
                if(target === "document"){
                    injectDocumentScript(details.tabId, f, details.frameId,domain)
                }else{
                    injectContentScript(details.tabId, f, details.frameId,domain)
                }
            });
            cssArray.forEach(f => {
                if(target === "document"){
                    injectDocumentCSS(details.tabId, f, details.frameId,domain)
                }else{
                    injectContentCSS(details.tabId, f, details.frameId,domain)
                }
            });
        });
    }
}
var jsonInject = function(details,code,domain,target){
    matchAndInject(code,details,(jsArray, cssArray)=>{
        jsArray.forEach(f => {
            if(target === "document"){
                injectDocumentScript(details.tabId, f, details.frameId,domain)
            }else{
                injectContentScript(details.tabId, f, details.frameId,domain)
            }
        });
        cssArray.forEach(f => {
            if(target === "document"){
                injectDocumentCSS(details.tabId, f, details.frameId,domain)
            }else{
                injectContentCSS(details.tabId, f, details.frameId,domain)
            }
        });
    });
}

var fetchSubMap = function(list,callback){
    let pList = [],cList = [];
    for (let i = 0; i < list.length; i++) {
        const e = list[i];
        for (let k = 0; k < e.domainLib.length; k++) {
            let g = e.domainLib[k];
            /* if(g.indexOf('http') === -1){
                g = urlPrex + g;
            } */
            cList.push({
                url:g,
                type:e.type,
                target:e.target,
                frame:e.frame || false,
                code:'',//预设，后续会赋值
            });
        }
    }

    cList.forEach(a=>{pList.push(action(a.url))});
    Promise.all(pList).then(rList=>{
        for (let j = 0; j < rList.length; j++) {
            const f = rList[j];
            cList[j].code = f;
        }
        callback && callback(cList);
    });
    function action(url){
        if(!url.includes('http')){
            url = urlPrex + url;
            // url = "http://localhost/"+url;
        }
        return new Promise((resolve)=>{
            get(url,function(code){
                resolve(code);
            })
        });
    }
}

/* 获取扩展名 */
var getExt = function(str){
    if(str.includes('?')){
        str = str.substr(0,str.indexOf('?'));
    }
    if(str.includes('.')){
        let list = str.split('.');
        const ext = list[list.length - 1].trim();
        if(['js','css'].includes(ext)){ return ext; }
    }
    return '';
}